/*
 * Copyright (C) 2010-2020 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(SERVICE_WORKER)

#include "ArgumentCoders.h"
#include "Connection.h"
#include "MessageNames.h"
#include "WebSWContextManagerConnectionMessagesReplies.h"
#include <WebCore/FetchIdentifier.h>
#include <WebCore/ServiceWorkerClientData.h>
#include <WebCore/ServiceWorkerData.h>
#include <WebCore/ServiceWorkerIdentifier.h>
#include <WebCore/ServiceWorkerTypes.h>
#include <wtf/Forward.h>
#include <wtf/Optional.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace IPC {
class FormDataReference;
}

namespace WebCore {
class ResourceRequest;
struct FetchOptions;
struct MessageWithMessagePorts;
struct ServiceWorkerContextData;
}

namespace WebKit {
struct WebPreferencesStore;
}

namespace Messages {
namespace WebSWContextManagerConnection {

static inline IPC::ReceiverName messageReceiverName()
{
    return IPC::ReceiverName::WebSWContextManagerConnection;
}

class InstallServiceWorker {
public:
    typedef std::tuple<const WebCore::ServiceWorkerContextData&, const String&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_InstallServiceWorker; }
    static const bool isSync = false;

    InstallServiceWorker(const WebCore::ServiceWorkerContextData& contextData, const String& userAgent)
        : m_arguments(contextData, userAgent)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class StartFetch {
public:
    typedef std::tuple<const WebCore::SWServerConnectionIdentifier&, const WebCore::ServiceWorkerIdentifier&, const WebCore::FetchIdentifier&, const WebCore::ResourceRequest&, const WebCore::FetchOptions&, const IPC::FormDataReference&, const String&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_StartFetch; }
    static const bool isSync = false;

    StartFetch(const WebCore::SWServerConnectionIdentifier& serverConnectionIdentifier, const WebCore::ServiceWorkerIdentifier& serviceWorkerIdentifier, const WebCore::FetchIdentifier& fetchIdentifier, const WebCore::ResourceRequest& request, const WebCore::FetchOptions& options, const IPC::FormDataReference& requestBody, const String& referrer)
        : m_arguments(serverConnectionIdentifier, serviceWorkerIdentifier, fetchIdentifier, request, options, requestBody, referrer)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CancelFetch {
public:
    typedef std::tuple<const WebCore::SWServerConnectionIdentifier&, const WebCore::ServiceWorkerIdentifier&, const WebCore::FetchIdentifier&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_CancelFetch; }
    static const bool isSync = false;

    CancelFetch(const WebCore::SWServerConnectionIdentifier& serverConnectionIdentifier, const WebCore::ServiceWorkerIdentifier& serviceWorkerIdentifier, const WebCore::FetchIdentifier& fetchIdentifier)
        : m_arguments(serverConnectionIdentifier, serviceWorkerIdentifier, fetchIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ContinueDidReceiveFetchResponse {
public:
    typedef std::tuple<const WebCore::SWServerConnectionIdentifier&, const WebCore::ServiceWorkerIdentifier&, const WebCore::FetchIdentifier&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_ContinueDidReceiveFetchResponse; }
    static const bool isSync = false;

    ContinueDidReceiveFetchResponse(const WebCore::SWServerConnectionIdentifier& serverConnectionIdentifier, const WebCore::ServiceWorkerIdentifier& serviceWorkerIdentifier, const WebCore::FetchIdentifier& fetchIdentifier)
        : m_arguments(serverConnectionIdentifier, serviceWorkerIdentifier, fetchIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PostMessageToServiceWorker {
public:
    typedef std::tuple<const WebCore::ServiceWorkerIdentifier&, const WebCore::MessageWithMessagePorts&, const WebCore::ServiceWorkerOrClientData&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_PostMessageToServiceWorker; }
    static const bool isSync = false;

    PostMessageToServiceWorker(const WebCore::ServiceWorkerIdentifier& destinationIdentifier, const WebCore::MessageWithMessagePorts& message, const WebCore::ServiceWorkerOrClientData& sourceData)
        : m_arguments(destinationIdentifier, message, sourceData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class FireInstallEvent {
public:
    typedef std::tuple<const WebCore::ServiceWorkerIdentifier&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_FireInstallEvent; }
    static const bool isSync = false;

    explicit FireInstallEvent(const WebCore::ServiceWorkerIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class FireActivateEvent {
public:
    typedef std::tuple<const WebCore::ServiceWorkerIdentifier&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_FireActivateEvent; }
    static const bool isSync = false;

    explicit FireActivateEvent(const WebCore::ServiceWorkerIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class TerminateWorker {
public:
    typedef std::tuple<const WebCore::ServiceWorkerIdentifier&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_TerminateWorker; }
    static const bool isSync = false;

    explicit TerminateWorker(const WebCore::ServiceWorkerIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class FindClientByIdentifierCompleted {
public:
    typedef std::tuple<uint64_t, const Optional<WebCore::ServiceWorkerClientData>&, bool> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_FindClientByIdentifierCompleted; }
    static const bool isSync = false;

    FindClientByIdentifierCompleted(uint64_t clientIdRequestIdentifier, const Optional<WebCore::ServiceWorkerClientData>& data, bool hasSecurityError)
        : m_arguments(clientIdRequestIdentifier, data, hasSecurityError)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class MatchAllCompleted {
public:
    typedef std::tuple<uint64_t, const Vector<WebCore::ServiceWorkerClientData>&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_MatchAllCompleted; }
    static const bool isSync = false;

    MatchAllCompleted(uint64_t matchAllRequestIdentifier, const Vector<WebCore::ServiceWorkerClientData>& clientsData)
        : m_arguments(matchAllRequestIdentifier, clientsData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetUserAgent {
public:
    typedef std::tuple<const String&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_SetUserAgent; }
    static const bool isSync = false;

    explicit SetUserAgent(const String& userAgent)
        : m_arguments(userAgent)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UpdatePreferencesStore {
public:
    typedef std::tuple<const WebKit::WebPreferencesStore&> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_UpdatePreferencesStore; }
    static const bool isSync = false;

    explicit UpdatePreferencesStore(const WebKit::WebPreferencesStore& store)
        : m_arguments(store)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class Close {
public:
    typedef std::tuple<> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_Close; }
    static const bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetThrottleState {
public:
    typedef std::tuple<bool> Arguments;

    static IPC::MessageName name() { return IPC::MessageName::WebSWContextManagerConnection_SetThrottleState; }
    static const bool isSync = false;

    explicit SetThrottleState(bool isThrottleable)
        : m_arguments(isThrottleable)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

} // namespace WebSWContextManagerConnection
} // namespace Messages

#endif // ENABLE(SERVICE_WORKER)
